package dev.pfaff.altargui.texture;

import static dev.pfaff.altargui.texture.TexCoord.toCharExact;

public record TextureAtlasScale(byte xShiftByte, byte yShiftByte) {
	public static final int MAX_SIZE = 0x1_0000;

	public static TextureAtlasScale fromAtlasSize(int width, int height) {
		var xShift = Integer.numberOfTrailingZeros(width);
		var yShift = Integer.numberOfTrailingZeros(height);
		if (1 << xShift != width || 1 << yShift != height) {
			throw new IllegalArgumentException(STR."Texture atlas size must be a power of two, found \{width}x\{height}");
		}
		if (xShift > 16 || yShift > 16) {
			throw new IllegalArgumentException(STR."Texture atlas size must be at most \{MAX_SIZE}x\{MAX_SIZE}, found \{width}x\{height}");
		}
		return new TextureAtlasScale((byte) (16 - xShift), (byte) (16 - yShift));
	}

	public int xShift() {
		return Byte.toUnsignedInt(this.xShiftByte);
	}

	public int yShift() {
		//noinspection SuspiciousNameCombination
		return Byte.toUnsignedInt(this.yShiftByte);
	}

	public int atlasWidth() {
		return atlasToPixelsX(MAX_SIZE);
	}

	public int atlasHeight() {
		return atlasToPixelsY(MAX_SIZE);
	}

	public char pixelsToAtlasX(char x) {
		return toCharExact(pixelsToAtlasX((int) x));
	}

	public char pixelsToAtlasY(char y) {
		return toCharExact(pixelsToAtlasY((int) y));
	}

	public char atlasToPixelsX(char x) {
		return toCharExact(atlasToPixelsX((int) x));
	}

	public char atlasToPixelsY(char y) {
		return toCharExact(atlasToPixelsY((int) y));
	}

	public int pixelsToAtlasX(int x) {
		return x << xShift();
	}

	public int pixelsToAtlasY(int y) {
		return y << yShift();
	}

	public int atlasToPixelsX(int x) {
		return x >>> xShift();
	}

	public int atlasToPixelsY(int y) {
		return y >>> yShift();
	}
}
