package dev.pfaff.altargui.util;

import java.util.Objects;
import java.util.concurrent.CancellationException;
import java.util.concurrent.CompletionException;
import java.util.function.Consumer;
import java.util.function.Function;

import static dev.pfaff.altargui.util.BasicFuture.completedTask;

record CompletedBasicFuture<V>(V value) implements BasicFuture<V> {
	@Override
	public V join() throws CompletionException, CancellationException {
		return this.value;
	}

	@Override
	public <U> BasicFuture<U> map(Function<V, U> mapper) {
		return completedTask(mapper.apply(this.value));
	}

	@Override
	public <U> BasicFuture<U> flatMap(Function<V, BasicFuture<U>> mapper) {
		Objects.requireNonNull(mapper);
		return completedTask(mapper.apply(this.value).cancelOnInterrupt().join());
	}

	@Override
	public BasicFuture<Void> thenAccept(Consumer<V> consumer) {
		consumer.accept(this.value);
		return COMPLETED_NULL;
	}
}
